/*
 * (C) Copyright 1992 by Simson Garfinkel and Associates, Inc.
 *
 * All Rights Reserved.
 *
 * Use of this module is covered by your source-code license agreement with
 * Simson Garfinkel and Associates, Inc.  Use of this module, or any code
 * that it contains, without a valid source-code license agreement is a
 * violation of copyright law.
 *
 */

/*
 * match.m:
 *
 */

#include "libsbook.h"
#include <ctype.h>

/* Return true if the word has an inner-cap */
bool isInnerCap(NXAtom str)
{
    unsigned int i;
    unsigned len = strlen(str);
    for(i=0;i<len-1;i++){
	if(islower(str[i]) && isupper(str[i+1])){
	    return true;
	}
    }
    return false;
}

/* Return true if the word looks like a title or roman numeral.
 * This should be replaced with something in flex.
 */
bool isRomanOrTitle(const QString &str)
{
    if(str.compare("II")==0) return true;
    if(str.compare("III")==0) return true;
    if(str.compare("IV")==0) return true;
    if(str.compare("V")==0) return true;
    if(str.compare("VI")==0) return true;
    if(str.compare("VII")==0) return true;
    if(str.compare("VIII")==0) return true;
    if(str.compare("IX")==0) return true;
    if(str.compare("X")==0) return true;
    if(str.compare("JR")==0) return true;
    if(str.compare("SR")==0) return true;
    if(str.compare("MD")==0) return true;
    if(str.compare("DDS")==0) return true;
    if(str.compare("ESQ")==0) return true;
    return false;
}


bool isOnlyDigits(const char *str)
{
    while(*str){
	if(!isdigit(*str)) return false;
	str++;
    }
    return true;
}

NXAtomList &atomsForNames(const QString &str)
{
    NXAtomList *names   = new NXAtomList;
    
    //const   unsigned char	*lastWord = 0;
    bool    inWord = false;
    int	    length = str.length();
    int	    i;
    int	    lastWordStart=0;

    for(i=0;i<length;i++){
	QChar cc = str.at(i);
	bool   isWordChar = cc.isLetter() || cc.isNumber() || cc=='\'' || cc=='*';

	if(cc=='.'){
	    QString upper4 = str.mid(i,4).upper();
	    if(upper4.compare(".COM")==0 ||
	       upper4.compare(".NET")==0 ||
	       upper4.compare(".ORG")==0){
		isWordChar=true;
	    }
	}


	if(inWord==false && isWordChar){
	    lastWordStart = i;
	    inWord = true;
	}
	if(inWord==true && ((i==length-1) || !isWordChar)){
	    QString word = str.mid(lastWordStart,i-lastWordStart + (isWordChar?1:0));

	    names->append(NXUniqueString(word.upper().latin1()));
	    inWord = false;
	}
    }
    return *names;
}
	
/* smartSortName:
 * Returns the name that we are sorting upon.
 */

NXAtom smartSortName(const char *theLine,NXAtomList &atoms)
{
    const int pflag = P_ADDRESS | P_COUNTRY | P_CITY | P_STATE | P_ORG;
    
    bool    hasComma = strchr(theLine,',');

    if(parse_company(theLine)
	|| (parse_address(theLine) & pflag)){
	/* It's a company.
	 * If the first word is the definate article, don't sort on it.
	 */
	//int j = atoms.count();
	if(atoms.count()>1
	   && atoms[0]==atom_THE){
	    return atoms[1];
	}

	/* Very special case.  if we are U. S. <something> return US... */
	if(atoms.count()>3 && atoms[0]==atom_U && atoms[1]==atom_S){
	    return atom_US;
	}
	return atoms[0];	/* a company */
    }

    /* If there is the word "and" that is not the first or last, and it is
     * not preeceded with a Mr. or Mrs., then this is probably a company,
     * so sort on the last name...
     */
    unsigned int i;
    for(i=1;i<atoms.count()-1;i++){
	if(atoms[i]==atom_AND){
	    /* If there is a comma, return the first name */
	    if(hasComma) return atoms[0];

	    /* If it is "x and y", return the first */
	    if(atoms.count()==3) return atoms[0];
	    
	    /* otherwise, return on the last name */
	    return atoms.last();
	}
    }

    /* If the first word is an innercap word, then it must be the
     * the sortname
     */
    if(strlen(atoms[0])>3 && isInnerCap(atoms[0])){
	return atoms[0];
    }
	

    /* If we find a comma, grab the word before it, even if that
     * word isn't one of the atoms...
     */
    if(hasComma){
	QString str(theLine);
	int comma = str.find(',');
	int before = str.findRev(' ',comma);
	if(before==-1) before=0;		  // start at beginning
	else	       before+=1;		  // skip past space
	QString ret = str.mid(before,comma-before).upper();
	return NXUniqueString(ret.latin1());
    }
		
    /* Otherwise, return the last word that isn't a single-character
     * and isn't a roman numeral or a title.
     */
    for(i=atoms.count()-1;i>0;i--){
	if(atoms[i][1]==0) continue;
	if(isRomanOrTitle(atoms[i])) continue;
	return atoms[i];
    }
    /* Return the first */
    return atoms[i];
}

