%{
/* Time parser built entirely out of flex */

extern "C" {
#include <time.h>
}

enum {
	AMERICAN,
	EUROPEAN
};
static 	int	year,month,day,hour,min,sec;
static	long	gmtoff;
static	char	*lptr;
int	date_format = AMERICAN;

#define	MIN(x,y) ((x)<(y) ? (x) : (y))
#undef  YY_INPUT
#define	YY_INPUT(buf,result,max_size)\
		result=MIN(strlen(lptr),max_size);\
		memcpy(buf,lptr,result);\
		lptr+=result;
#undef 	ECHO
#define	ECHO {}
			
%}

YEAR	(([0-9][0-9])?([0-9][0-9]))
NUM 	([0-9][0-9]?)
NN 	([^0-9])

JAN	([jJ][aA][nN]([uU][aA][rR][yY])?)
FEB	([fF][eE][bB]([rR][uU][aA][rR][yY])?)
MAR	([mM][aA][rR]([cC][hH])?)
APR	([aA][pP][rR]([iI][lL])?)
MAY	([mM][aA][yY])
JUN	([jJ][uU][nN]([eE])?)
JUL	([jJ][uU][lL]([yY])?)
AUG	([aA][uU][gG]([uU][sS][tT])?)
SEP	([sS][eE][pP]([tT][eE][mM][bB][eE][rR])?)
OCT	([oO][cC][tT]([oO][bB][eE][rR])?)
NOV	([nN][oO][vV]([eE][mM][bB][eE][rR])?)
DEC	([dD][eE][cC]([eE][mM][bB][eE][rR])?)

AMONTH	({JAN}|{FEB}|{MAR}|{APR}|{MAY}|{JUN}|{JUL}|{AUG}|{SEP}|{OCT}|{NOV}|{DEC})


DATE1	({NUM}-{AMONTH}-{YEAR})
DATE2	({AMONTH}[ \t]*{NUM}[ ,\t]*{YEAR})

%%

{DATE1}|{DATE2}			{
				char	*cc;

				month	= parse_month(yytext);
				for(cc=yytext;*cc && !isdigit(*cc);cc++){
				}
				day	= atoi(cc);
				for(;*cc && isdigit(*cc);cc++){
				}
				for(;*cc && !isdigit(*cc);cc++){
				}
				year	= atoi(cc);
				}
				

{NUM}[ \t]*{AMONTH}[ \t]*{YEAR}	{
				char *cc;

				for(cc=yytext+strlen(yytext)-1;
					isdigit(*cc);
					cc--){
				}	
				day	= atoi(yytext);
				year	= atoi(cc+1);
				month	= parse_month(yytext);
				}

{NUM}"/"{NUM}"/"{YEAR}		{
				int first,second;
				sscanf(yytext,"%d/%d/%d",&first,&second,&year);
				if(first>12)	date_format = EUROPEAN;
				if(second>12)	date_format = AMERICAN;
				if(date_format==AMERICAN){
					month	= first - 1;
					day	= second;
				}
				else{
					month 	= second - 1;
					day	= first;
				}
				}


{NUM}":"{NUM}(":"{NUM})?[ \t]*[aA]/{NN}	{
				/* handle a time with an a.m.  We have to
				 * special case 12:01 a.m. and the linke
				 */
				sscanf(yytext,"%d:%d:%d",&hour,&min,&sec);
				if(hour==12) hour=0;
				}

{NUM}":"{NUM}(":"{NUM})?[ \t]*[pP]/{NN}	{
				/* handle a time with a p.m.
				 */
				sscanf(yytext,"%d:%d:%d",&hour,&min,&sec);
				hour+=12;
				}

{NUM}":"{NUM}(":"{NUM})?/{NN}	{sscanf(yytext,"%d:%d:%d",&hour,&min,&sec);}

"GMT"?("+"|"-")[0-9][0-9][0-9][0-9]	{
				char	c;
				int	offhours,offmin;

				sscanf(yytext,"%c%2d%2d",
					&c,&offhours,&offmin);

				gmtoff	= (offhours*60+offmin)*60;
				if(c=='-') gmtoff = -gmtoff;
				}

[0-9][0-9][0-9][0-9]"-"/([A-Z][SD]"T")	{
				sscanf(yytext,"%2d%2d",&hour,&min);
				}
				

[jJeEpPmMcCbB][sSdD][tT]		{
				switch(toupper(yytext[0])){
					case 'E': gmtoff = -5; break;
					case 'C': gmtoff = -6; break;
					case 'M': gmtoff = -7; break;
					case 'P': gmtoff = -8; break;
					case 'B': gmtoff = -8; break;
						/* Berkeley */
					case 'J': gmtoff =  9; break;
						/* Japan    */
				}
				if(toupper(yytext[1])=='D') gmtoff -= 1;
				gmtoff  *= 60*60;
			
				}

.						{/* null rule --- ignore */}
%%
void	parse_time(char *buf,struct tm *tm)
{
	char	*alloca(int size);
	char	*mbuf	= alloca(strlen(buf)+2);

	strcpy(mbuf,buf);
	strcat(mbuf," ");

	lptr	= mbuf;
	year 	= month = day = hour = min = sec = -1;
	gmtoff	= -1L;

	/* initialize flex static variables */
	yy_current_buffer	= 0;
	yy_c_buf_p 		= (YY_CHAR *)0;
	yy_init			= 1;		/* be sure to init */
	yy_start		= 0;
	yyin			= (FILE *)0;
	yyout			= (FILE *)0;
	yylex();

	if(year>1900){
		year -= 1900;
	}

	if(year 	!= -1)	tm->tm_year	= year;
	if(month	!= -1)	tm->tm_mon	= month;
	if(day  	!= -1)	tm->tm_mday	= day;
	if(hour 	!= -1)  tm->tm_hour	= hour;
	if(min		!= -1)	tm->tm_min	= min;
	if(sec		!= -1)	tm->tm_sec	= sec;
	if(gmtoff	!= -1L) tm->tm_gmtoff	= gmtoff;
	return;
}

